#include <vector>
#include "testlib.h"

const int MAX_GRAPH_SIZE = 5000;

enum VertexType
{
  Input, And, Or, Not, Output
};

struct Vertex
{
  VertexType type;
  int inputs[2];
  int computed_value;
  
  Vertex(VertexType type, int input1, int input2, int computed_value) :
	type(type), computed_value(computed_value)
  {
	  inputs[0] = input1;
	  inputs[1] = input2;
  }
};

void read_vertex(InStream& input, Vertex& vertex, int k)
{
  char type = input.readChar();
  if (type == '|')
  {
    vertex = Vertex(Or, input.readInt(1, k) - 1, input.readInt(1, k) - 1, -1);
  }
  else if (type == '&')
  {
    vertex = Vertex(And, input.readInt(1, k) - 1, input.readInt(1, k) - 1, -1);
  }
  else if (type == '!')
  {
    vertex = Vertex(Not, input.readInt(1, k) - 1, -1, -1);
  }
  else if (type == '#')
  {
    vertex = Vertex(Output, input.readInt(1, k) - 1, -1, -1);
  }
  else
  {
    quitf(_wa, "character '%c' does not name vertex type", type);
  }
  input.seekEoln();
}

std::vector<Vertex> read_participants_output(int n, int k, int *output_index)
{
  std::vector<Vertex> vertices(k, Vertex(Input, -1, -1, -1));
  int number_of_diagram_outputs = 0;
  std::vector<int> outputs_counts(k, 0);
  for (int i = 0; i < k - n; ++i)
  {
    read_vertex(ouf, vertices[i + n], k);
    if (vertices[i + n].type == Output)
    {
      number_of_diagram_outputs++;
      *output_index = i + n;
    }
    for (int q = 0; q < 2; ++q)
    {
      if (vertices[i + n].inputs[q] >= 0)
      {
        outputs_counts[vertices[i + n].inputs[q]]++;
      }
    }
  }
  if (number_of_diagram_outputs != 1)
  {
    quitf(_wa, "number of outputs in the diagram does not equal one");
  }
  if (outputs_counts[*output_index] > 0)
  {
    quitf(_wa, "some element in the diagram takes input from diagram's output");
  }
  return vertices;
}


int compute_participants_answer(int current,
    std::vector<Vertex>& vertices, std::vector<int>& visited)
{
  if (visited[current] == 1)
  {
    return -1;
  }
  visited[current] = 1;
  int inputs[] = { -1, -1 };
  for (int q = 0; q < 2; ++q)
  {
    if (vertices[current].inputs[q] >= 0)
    {
      if (visited[vertices[current].inputs[q]] == 2)
      {
        inputs[q] = vertices[vertices[current].inputs[q]].computed_value;
      }
      else
      {
        inputs[q] = compute_participants_answer(
            vertices[current].inputs[q], vertices, visited);
      }
      if (inputs[q] < 0)
      {
        return -1;
      }
    }
  }
  visited[current] = 2;
  switch (vertices[current].type)
  {
    case Input: return vertices[current].computed_value;
    case And: return vertices[current].computed_value = inputs[0] & inputs[1];
    case Or: return vertices[current].computed_value = inputs[0] | inputs[1];
    case Not: return vertices[current].computed_value = 1 - inputs[0];
    case Output: return vertices[current].computed_value = inputs[0];
  }
}

int popcount(int x)
{
  int result = 0;
  while (x > 0)
  {
    result += (x % 2);
    x /= 2;
  }
  return result;
}

std::string to_binary(int bitmask, int number_of_inputs)
{
  std::string result(number_of_inputs * 2 - 1, ' ');
  for (int i = 1, j = 0; j < number_of_inputs; i *= 2, j++)
  {
    result[j * 2] = ('0' + ((i & bitmask) > 0));
  }
  return result;
}


int main(int argc, char **argv)
{
  registerTestlibCmd(argc, argv);

  int n = inf.readInt(1, 5000);
  int k = ouf.readInt(n, MAX_GRAPH_SIZE, "k");
  ouf.seekEoln();
  int output_index = -1;

  read_participants_output(n, k, &output_index);

  quitf(_ok, "diagram in the output is ok");
  return 0;
}
